using System;
using System.Collections.Specialized;
using Edfinancial.SharePoint.STSADM.Commands.SPValidators;
using Microsoft.SharePoint;

namespace Edfinancial.SharePoint.STSADM.Commands.Lists
{
    public class DeleteList : SPOperation
    {
         /// <summary>
        /// Initializes a new instance of the <see cref="DeleteList"/> class.
        /// </summary>
        public DeleteList()
        {
            SPParamCollection parameters = new SPParamCollection();
            parameters.Add(new SPParam("url", "url", true, null, new SPUrlValidator(), "Please specify the list view URL."));
            parameters.Add(new SPParam("force", "f"));
            Init(parameters, "\r\n\r\nDeletes a list.\r\n\r\nParameters:\r\n\t-url <list view URL>\r\n\t[-force]");
        }

        #region ISPStsadmCommand Members

        /// <summary>
        /// Gets the help message.
        /// </summary>
        /// <param name="command">The command.</param>
        /// <returns></returns>
        public override string GetHelpMessage(string command)
        {
            return HelpMessage;
        }

        /// <summary>
        /// Runs the specified command.
        /// </summary>
        /// <param name="command">The command.</param>
        /// <param name="keyValues">The key values.</param>
        /// <param name="output">The output.</param>
        /// <returns></returns>
        public override int Run(string command, StringDictionary keyValues, out string output)
        {
            output = string.Empty;

            InitParameters(keyValues);

            string url = Params["url"].Value;
            bool force = Params["force"].UserTypedIn;
            SPList list = Utilities.GetListFromViewUrl(url);

            Delete(list, force, url);

            return 1;
        }

        #endregion

        /// <summary>
        /// Deletes the specified list.
        /// </summary>
        /// <param name="list">The list.</param>
        /// <param name="force">if set to <c>true</c> [force].</param>
        /// <param name="url">The URL.</param>
        internal static void Delete(SPList list, bool force, string url)
        {
            if (list == null)
                throw new SPException("List not found.");

            if (!list.AllowDeletion && force)
            {
                list.AllowDeletion = true;
                list.Update();
            }
            else if (!list.AllowDeletion)
                throw new SPException("List cannot be deleted.  Try using the '-force' parameter to force the delete.");

            try
            {
                list.Delete();
            }
            catch (Exception)
            {
                if (force)
                {
                    using (SPSite site = new SPSite(url))
                    {
                        Utilities.RunStsAdmOperation(
                            string.Format(" -o forcedeletelist -url \"{0}\"",
                                          site.MakeFullUrl(list.RootFolder.ServerRelativeUrl)), false);
                    }
                }
            }
        }
    }
}
