using System;
using System.Collections.Generic;
using System.Collections.Specialized;
using System.Text;
using Edfinancial.SharePoint.STSADM.Commands.SPValidators;
using Microsoft.SharePoint;

namespace Edfinancial.SharePoint.STSADM.Commands.Lists
{
    public class DeleteListItem : SPOperation
    {
        /// <summary>
        /// Initializes a new instance of the <see cref="DeleteListItem"/> class.
        /// </summary>
        public DeleteListItem()
        {
            SPParamCollection parameters = new SPParamCollection();
            parameters.Add(new SPParam("url", "url", true, null, new SPUrlValidator(), "Please specify the list view URL."));
            parameters.Add(new SPParam("id", "id", false, null, new SPNonEmptyValidator()));
            parameters.Add(new SPParam("deletefolders", "df"));

            StringBuilder sb = new StringBuilder();
            sb.Append("\r\n\r\nDeletes an item or items from a list.\r\n\r\nParameters:");
            sb.Append("\r\n\t-url <list view URL>");
            sb.Append("\r\n\t[-id <list item ID (separate multiple items with a comma)>]");
            sb.Append("\r\n\t[-deletefolders]");
            Init(parameters, sb.ToString());
        }

        #region ISPStsadmCommand Members

        /// <summary>
        /// Gets the help message.
        /// </summary>
        /// <param name="command">The command.</param>
        /// <returns></returns>
        public override string GetHelpMessage(string command)
        {
            return HelpMessage;
        }

        /// <summary>
        /// Runs the specified command.
        /// </summary>
        /// <param name="command">The command.</param>
        /// <param name="keyValues">The key values.</param>
        /// <param name="output">The output.</param>
        /// <returns></returns>
        public override int Run(string command, StringDictionary keyValues, out string output)
        {
            output = string.Empty;

            InitParameters(keyValues);

            string url = Params["url"].Value;
            List<int> ids = new List<int>();
            if (Params["id"].UserTypedIn)
            {
                string[] ids2 = Params["id"].Value.Split(',');
                ids = new List<int>(ids2.Length);
                for (int i = 0; i < ids2.Length; i++)
                    if (!string.IsNullOrEmpty(ids2[i]))
                        ids.Add(int.Parse(ids2[i]));
            }

            DeleteListItems(url, ids, Params["deletefolders"].UserTypedIn);

            return 1;
        }

        /// <summary>
        /// Deletes the list items.
        /// </summary>
        /// <param name="url">The URL.</param>
        /// <param name="ids">The ids.</param>
        /// <param name="deleteFolders">if set to <c>true</c> [delete folders].</param>
        internal static void DeleteListItems(string url, List<int> ids, bool deleteFolders)
        {
            using (SPSite site = new SPSite(url))
            {
                using (SPWeb web = site.OpenWeb())
                {
                    SPList list = Utilities.GetListFromViewUrl(web, url);

                    if (list == null)
                    {
                        throw new Exception("List not found.");
                    }

                    if (ids.Count == 0)
                    {
                        while (true)
                        {
                            if (list.Items.Count == 0)
                                break;

                            SPListItem item = list.Items[0];
                            item.Delete();
                        }
                        if (deleteFolders)
                        {
                            while (true)
                            {
                                if (list.Folders.Count == 0)
                                    break;

                                list.Folders[0].Delete();
                            }
                        }
                    }
                    else
                    {
                        foreach (int id in ids)
                        {
                            SPListItem item = null;
                            try
                            {
                                item = list.GetItemById(id);
                                item.Delete();
                            }
                            catch (ArgumentException)
                            {
                            }
                            if (item == null)
                                Console.WriteLine(string.Format("WARNING - List item not found: {0}.", id));
                        }
                    }
                }
            }
        }

        #endregion
    }
}
