using System;
using System.Collections.Specialized;
using Edfinancial.SharePoint.STSADM.Commands.SPValidators;
using Microsoft.SharePoint;

namespace Edfinancial.SharePoint.STSADM.Commands.Lists
{
    public class DeleteListView : SPOperation
    {
         /// <summary>
        /// Initializes a new instance of the <see cref="DeleteListView"/> class.
        /// </summary>
        public DeleteListView()
        {
            SPParamCollection parameters = new SPParamCollection();
            parameters.Add(new SPParam("url", "url", true, null, new SPUrlValidator(), "Please specify the list view URL."));
            Init(parameters, "\r\n\r\nDeletes a view from a list.\r\n\r\nParameters:\r\n\t-url <list view URL>");
        }

        #region ISPStsadmCommand Members

        /// <summary>
        /// Gets the help message.
        /// </summary>
        /// <param name="command">The command.</param>
        /// <returns></returns>
        public override string GetHelpMessage(string command)
        {
            return HelpMessage;
        }

        /// <summary>
        /// Runs the specified command.
        /// </summary>
        /// <param name="command">The command.</param>
        /// <param name="keyValues">The key values.</param>
        /// <param name="output">The output.</param>
        /// <returns></returns>
        public override int Run(string command, StringDictionary keyValues, out string output)
        {
            output = string.Empty;

            InitParameters(keyValues);

            string url = Params["url"].Value;

            using (SPSite site = new SPSite(url))
            {
                using (SPWeb web = site.OpenWeb())
                {
                    SPList list = Utilities.GetListFromViewUrl(web, url);

                    if (list == null)
                    {
                        throw new Exception("List not found.");
                    }

                    SPView viewToDelete = null;
                    foreach (SPView view in list.Views)
                    {
                        if (url.ToLower() == web.Site.MakeFullUrl(view.ServerRelativeUrl).ToLower())
                        {
                            viewToDelete = view;
                            break;
                        }
                    }

                    if (viewToDelete == null)
                    {
                        throw new Exception("View not found.");
                    }

                    list.Views.Delete(viewToDelete.ID);
                }
            }
            return 1;
        }

        #endregion
    }
}
