using System;
using System.Collections.Specialized;
using System.IO;
using System.Text;
using System.Xml;
using Edfinancial.SharePoint.STSADM.Commands.SPValidators;
using Microsoft.SharePoint;

namespace Edfinancial.SharePoint.STSADM.Commands.Lists
{
    public class GetListSchemaXml : SPOperation
    {
        /// <summary>
        /// Initializes a new instance of the <see cref="GetListSchemaXml"/> class.
        /// </summary>
        public GetListSchemaXml()
        {
            SPParamCollection parameters = new SPParamCollection();
            parameters.Add(new SPParam("url", "url", true, null, new SPUrlValidator(), "Please specify the list url."));
            Init(parameters, "\r\n\r\nReturns the XML schema for a list.\r\n\r\nParameters:\r\n\t-url <list view url>");
        }

        #region ISPStsadmCommand Members

        /// <summary>
        /// Gets the help message.
        /// </summary>
        /// <param name="command">The command.</param>
        /// <returns></returns>
        public override string GetHelpMessage(string command)
        {
            return HelpMessage;
        }

        /// <summary>
        /// Runs the specified command.
        /// </summary>
        /// <param name="command">The command.</param>
        /// <param name="keyValues">The key values.</param>
        /// <param name="output">The output.</param>
        /// <returns></returns>
        public override int Run(string command, StringDictionary keyValues, out string output)
        {
            output = string.Empty;

            InitParameters(keyValues);

            string url = Params["url"].Value;

            using (SPSite site = new SPSite(url))
            {
                using (SPWeb web = site.OpenWeb())
                {
                    SPList list = Utilities.GetListFromViewUrl(web, url);

                    if (list == null)
                    {
                        throw new Exception("List not found.");
                    }

                    // Load the xml into an XmlDocument object so that we can write it out with formatting applied (probably an easier way to do this...).
                    XmlDocument xmlDoc = new XmlDocument();
                    xmlDoc.LoadXml(list.SchemaXml);

                    StringBuilder sb = new StringBuilder();
                    XmlTextWriter xmlWriter = new XmlTextWriter(new StringWriter(sb));
                    xmlWriter.Formatting = Formatting.Indented;
                    xmlDoc.WriteContentTo(xmlWriter);
                    xmlWriter.Flush();

                    output += sb.ToString();
                }
            }
            return 1;
        }

        #endregion
    }
}
