using System;
using System.Collections.Specialized;
using System.Text;
using Edfinancial.SharePoint.STSADM.Commands.SPValidators;
using Microsoft.Office.Server;
using Microsoft.Office.Server.UserProfiles;

namespace Edfinancial.SharePoint.STSADM.Commands.MySites
{
    public class SetSiteNamingFormat : SPOperation
    {
        /// <summary>
        /// Initializes a new instance of the <see cref="SetSiteNamingFormat"/> class.
        /// </summary>
        public SetSiteNamingFormat()
        {
            SPParamCollection parameters = new SPParamCollection();
            parameters.Add(new SPParam("sspname", "ssp", true, null, new SPNonEmptyValidator(), "Please specify the SSP name."));
            parameters.Add(
                new SPParam("format", "f", true, null,
                            new SPRegexValidator("(?i:^Username_CollisionError$|^Username_CollisionDomain$|^Domain_Username$)")));
            StringBuilder sb = new StringBuilder();
            sb.Append("\r\n\r\nSets the site naming format for the My Sites web application.\r\n\r\nParameters:");
            sb.Append("\r\n\t-sspname <SSP name>");
            sb.Append("\r\n\t-format <Username_CollisionError | Username_CollisionDomain | Domain_Username>");
            Init(parameters, sb.ToString());
        }

        #region ISPStsadmCommand Members

        /// <summary>
        /// Gets the help message.
        /// </summary>
        /// <param name="command">The command.</param>
        /// <returns></returns>
        public override string GetHelpMessage(string command)
        {
            return HelpMessage;
        }

        /// <summary>
        /// Runs the specified command.
        /// </summary>
        /// <param name="command">The command.</param>
        /// <param name="keyValues">The key values.</param>
        /// <param name="output">The output.</param>
        /// <returns></returns>
        public override int Run(string command, StringDictionary keyValues, out string output)
        {
            output = string.Empty;

            InitParameters(keyValues);

            string sspname = Params["sspname"].Value;
            string format = Params["format"].Value;

            SiteNameFormat nameFormat = (SiteNameFormat)Enum.Parse(typeof(SiteNameFormat), format, true);

            UserProfileManager upm = new UserProfileManager(ServerContext.GetContext(sspname));

            MySiteSettings.CommitPersonalSiteSettings(upm, upm.IsPersonalSiteMultipleLanguage, upm.PersonalSiteInclusion,
                                                      nameFormat, upm.PersonalSiteReaders);

            return 1;
        }

        #endregion
    }
}
