using System;
using System.Collections.Generic;
using System.Collections.Specialized;
using System.IO;
using System.Text;
using Edfinancial.SharePoint.STSADM.Commands.SPValidators;
using Microsoft.SharePoint;
using Microsoft.SharePoint.Publishing;
using Microsoft.SharePoint.Utilities;

namespace Edfinancial.SharePoint.STSADM.Commands.Pages
{
    public class SiteWelcomePage : SPOperation
    {
        /// <summary>
        /// Initializes a new instance of the <see cref="SiteWelcomePage"/> class.
        /// </summary>
        public SiteWelcomePage()
        {
            SPParamCollection parameters = new SPParamCollection();
            parameters.Add(new SPParam("site", "site", true, null, new SPUrlValidator()));
            parameters.Add(new SPParam("welcomepage", "page", true, null, new SPUrlValidator()));

            StringBuilder sb = new StringBuilder();
            sb.Append("\r\n\r\nSets the page to be used as the welcome page for the site.\r\n\r\nParameters:");
            sb.Append("\r\n\t-site <url of the site to update>");
            sb.Append("\r\n\t-welcomepage <full url to the page to use as the welcome page>");

            Init(parameters, sb.ToString());
        }

        #region ISPStsadmCommand Members

        /// <summary>
        /// Gets the help message.
        /// </summary>
        /// <param name="command">The command.</param>
        /// <returns></returns>
        public override string GetHelpMessage(string command)
        {
            return HelpMessage;
        }

        /// <summary>
        /// Runs the specified command.
        /// </summary>
        /// <param name="command">The command.</param>
        /// <param name="keyValues">The key values.</param>
        /// <param name="output">The output.</param>
        /// <returns></returns>
        public override int Run(string command, StringDictionary keyValues, out string output)
        {
            output = string.Empty;

            InitParameters(keyValues);

            string url = Params["site"].Value.TrimEnd('/');
            string page = SPHttpUtility.UrlPathDecode(Params["welcomepage"].Value, true);

            using (SPSite site = new SPSite(url))
            using (SPWeb web = site.AllWebs[Utilities.GetServerRelUrlFromFullUrl(url)])
            {
                if (!PublishingWeb.IsPublishingWeb(web))
                    throw new SPException("The specified site is not a publishing web.");

                PublishingWeb pubWeb = PublishingWeb.GetPublishingWeb(web);

                bool flag = false;
                SPFile file = null;
                try
                {
                    file = pubWeb.Web.GetFile(page);
                    flag = file.Exists;
                }
                catch (SPException)
                {
                    flag = false;
                }
                catch (FileNotFoundException)
                {
                    flag = false;
                }
                catch (ArgumentException)
                {
                    flag = false;
                }

                if (!flag || file == null)
                    throw new SPException("The specified welcome page could not be found.");


                PublishingWeb currentPublishingWeb = pubWeb;
                currentPublishingWeb.DefaultPage = file;
                currentPublishingWeb.Update();
            }

            return 1;
        }
        
        #endregion
    }
}
