using System;
using System.Collections.Specialized;
using Microsoft.SharePoint;
using Microsoft.SharePoint.StsAdmin;

namespace Edfinancial.SharePoint.STSADM.Commands
{
    public abstract class SPOperation : ISPStsadmCommand
    {
        private SPParamCollection m_params;
        private string m_helpMessage;


        // 98d3057cd9024c27b2007643c1 is a special hard coded name for a list that Microsoft uses to store the mapping
        // of URLs from v2 to v3 (maps the bucket urls to the new urls).
        protected const string UPGRADE_AREA_URL_LIST = "98d3057cd9024c27b2007643c1";


        /// <summary>
        /// Inits the specified parameters.
        /// </summary>
        /// <param name="parameters">The parameters.</param>
        /// <param name="helpMessage">The help message.</param>
        protected void Init(SPParamCollection parameters, string helpMessage)
        {
            helpMessage = helpMessage.TrimEnd(new char[] {'\r', '\n'});
#if DEBUG
            helpMessage += "\r\n\t[-debug]";
            parameters.Add(new SPParam("debug", "debug"));
#endif

            m_params = parameters;
            
            helpMessage +=
                "\r\n\r\n\r\nCopyright 2007 Edfinancial Services\r\n  > For more information on this command and others:\r\n  > http://stsadm.blogspot.com/\r\n  > Use of this command is at your own risk.\r\n  > Edfinancial Services assumes no liability.";
            m_helpMessage = helpMessage;
        }

        /// <summary>
        /// Inits the parameters.
        /// </summary>
        /// <param name="keyValues">The key values.</param>
        public virtual void InitParameters(StringDictionary keyValues)
        {
            foreach (SPParam param in Params)
            {
                param.InitValueFrom(keyValues);
            }
            Validate(keyValues);
#if DEBUG
            if (Params["debug"].UserTypedIn)
                System.Diagnostics.Debugger.Launch();
#endif
        }


        /// <summary>
        /// Validates the specified key values.
        /// </summary>
        /// <param name="keyValues">The key values.</param>
        public virtual void Validate(StringDictionary keyValues)
        {
            string strMessage = null;

            foreach (string current in keyValues.Keys)
            {
                if (current != "o" && Params[current] == null)
                {
                    strMessage += string.Format("Command line error. Invalid parameter: {0}.\r\n", current);
                }
            }
            if (strMessage != null)
                throw new SPSyntaxException(strMessage);

            foreach (SPParam param in Params)
            {
                if (param.Enabled)
                {
                    if (param.IsRequired && !param.UserTypedIn)
                    {
                        strMessage += SPResource.GetString("MissRequiredArg", new object[] { param.Name }) + "\r\n";
                    }
                }
            }
            if (strMessage != null)
                throw new SPSyntaxException(strMessage);

            foreach (SPParam param in Params)
            {
                if (param.Enabled)
                {
                    if (param.UserTypedIn && !param.Validate())
                    {
                        strMessage += SPResource.GetString("InvalidArg", new object[] { param.Name });
                        if (!string.IsNullOrEmpty(param.HelpMessage))
                        {
                            strMessage += "\r\n\t" + param.HelpMessage + "\r\n";
                        }
                    }
                }
            }
            if (strMessage != null)
                throw new SPSyntaxException(strMessage);
        }

        // Properties
        public virtual string DisplayNameId
        {
            get
            {
                return null;
            }
        }

       

        public string HelpMessage
        {
            get
            {
                return m_helpMessage;
            }
        }

        protected internal SPParamCollection Params
        {
            get
            {
                return m_params;
            }
        }

        #region ISPStsadmCommand Members

        public abstract string GetHelpMessage(string command);

        public abstract int Run(string command, StringDictionary keyValues, out string output);

        #endregion
    }

    internal class SPSyntaxException : ApplicationException
    {
        // Methods
        public SPSyntaxException(string strMessage)
            : base(strMessage)
        {
        }
    }

 

 
}
