using System;

namespace Edfinancial.SharePoint.STSADM.Commands.SPValidators
{
    internal class SPEnumValidator : SPRegexValidator
    {
        private Type enumTypeValue;
        private string[] additionalValues = null;

        /// <summary>
        /// Initializes a new instance of the <see cref="SPEnumValidator"/> class.
        /// </summary>
        /// <param name="enumType">Type of the enum.</param>
        public SPEnumValidator(Type enumType) : base(GetEnumRegex(enumType, null), SPRegexValidationType.Legal)
        {
            enumTypeValue = enumType;
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="SPEnumValidator"/> class.
        /// </summary>
        /// <param name="enumType">Type of the enum.</param>
        /// <param name="additionalValues">The additional values.</param>
        public SPEnumValidator(Type enumType, string[] additionalValues) : base(GetEnumRegex(enumType, additionalValues), SPRegexValidationType.Legal)
        {
            enumTypeValue = enumType;
            this.additionalValues = additionalValues;
        }

        /// <summary>
        /// Gets the enum regex.
        /// </summary>
        /// <param name="enumType">Type of the enum.</param>
        /// <param name="additionalValues">The additional values.</param>
        /// <returns></returns>
        private static string GetEnumRegex(Type enumType, string[] additionalValues)
        {
            if (!enumType.IsEnum)
                throw new Exception("Type is not an enum type.");

            string regex = string.Join("$|^", Enum.GetNames(enumType));
            if (additionalValues != null && additionalValues.Length > 0)
                regex += "$|^" + string.Join("$|^", additionalValues);

            regex = "(?i:^" + regex + "$)";
            return regex;
        }

        /// <summary>
        /// Gets the display value.
        /// </summary>
        /// <value>The display value.</value>
        internal string DisplayValue
        {
            get
            {
                string val = string.Join(" | ", Enum.GetNames(enumTypeValue));
                if (additionalValues != null && additionalValues.Length > 0)
                    val += " | " + string.Join(" | ", additionalValues);
                return val.ToLowerInvariant();
            }
        }
    }
}
