using System.Text;
using Edfinancial.SharePoint.STSADM.Commands.SPValidators;
using Microsoft.SharePoint;

namespace Edfinancial.SharePoint.STSADM.Commands.Search
{
    public class SetSearchCenter : SPOperation
    {
        /// <summary>
        /// Initializes a new instance of the <see cref="SetSearchCenter"/> class.
        /// </summary>
        public SetSearchCenter()
        {
            SPParamCollection parameters = new SPParamCollection();
            parameters.Add(new SPParam("url", "url", true, null, new SPUrlValidator(), "Please specify the site collection."));
            parameters.Add(new SPParam("searchcenter", "s", false, null, null));

            StringBuilder sb = new StringBuilder();
            sb.Append("\r\n\r\nSets the search center for a given site collection (settings can also be set via: \"{site collection}/_layouts/enhancedSearch.aspx\").\r\n\r\nParameters:");
            sb.Append("\r\n\t-url <site collection url>");
            sb.Append("\r\n\t-searchcenter <search center url (leave empty or ommit to use {site collection}/_layouts/osssearchresults.aspx)>");

            Init(parameters, sb.ToString());
        }

        #region ISPStsadmCommand Members
        /// <summary>
        /// Gets the help message.
        /// </summary>
        /// <param name="command">The command.</param>
        /// <returns></returns>
        public override string GetHelpMessage(string command)
        {
            return HelpMessage;
        }

        /// <summary>
        /// Runs the specified command.
        /// </summary>
        /// <param name="command">The command.</param>
        /// <param name="keyValues">The key values.</param>
        /// <param name="output">The output.</param>
        /// <returns></returns>
        public override int Run(string command, System.Collections.Specialized.StringDictionary keyValues, out string output)
        {
            output = string.Empty;

            InitParameters(keyValues);

            string url = Params["url"].Value.TrimEnd('/');

            using (SPSite site = new SPSite(url))
            using (SPWeb web = site.RootWeb)
            {
                string searchCenter = null;
                if (Params["searchcenter"].UserTypedIn)
                {
                    searchCenter = Params["searchcenter"].Value;
                    if (searchCenter != null && searchCenter.Trim() == "")
                        searchCenter = null;
                }

                web.AllProperties["SRCH_ENH_FTR_URL"] = searchCenter;
                web.Update();
            }

            return 1;
        }

        #endregion

    }
}
