using System.Collections.Specialized;
using System.Text;
using Edfinancial.SharePoint.STSADM.Commands.SPValidators;
using Microsoft.SharePoint;

namespace Edfinancial.SharePoint.STSADM.Commands.SiteCollectionSettings
{
    public class ConnectToPortalSite : SPOperation
    {
        /// <summary>
        /// Initializes a new instance of the <see cref="ConnectToPortalSite"/> class.
        /// </summary>
        public ConnectToPortalSite()
        {
            SPParamCollection parameters = new SPParamCollection();
            parameters.Add(new SPParam("url", "url", true, null, new SPUrlValidator(), "Please specify the site collection url."));
            parameters.Add(new SPParam("portalurl", "purl", false, null, new SPUrlValidator(), "Please specify the portal url."));
            parameters.Add(new SPParam("portalname", "name", false, null, new SPNonEmptyValidator(), "Please specify the portal name."));
            parameters.Add(new SPParam("disconnect", "d", false, null, null));
            
            StringBuilder sb = new StringBuilder();
            sb.Append("\r\n\r\nConnects a site collection to a portal.\r\n\r\nParameters:");
            sb.Append("\r\n\t-url <site collection>");
            sb.Append("\r\n\t[-portalurl <portal web address>]");
            sb.Append("\r\n\t[-portalname <portal name>]");
            sb.Append("\r\n\t[-disconnect]");
            Init(parameters, sb.ToString());
        }

        #region ISPStsadmCommand Members

        /// <summary>
        /// Gets the help message.
        /// </summary>
        /// <param name="command">The command.</param>
        /// <returns></returns>
        public override string GetHelpMessage(string command)
        {
            return HelpMessage;
        }

        /// <summary>
        /// Runs the specified command.
        /// </summary>
        /// <param name="command">The command.</param>
        /// <param name="keyValues">The key values.</param>
        /// <param name="output">The output.</param>
        /// <returns></returns>
        public override int Run(string command, StringDictionary keyValues, out string output)
        {
            output = string.Empty;

            InitParameters(keyValues);

            bool disconnect = Params["disconnect"].UserTypedIn;

            string url = Params["url"].Value.TrimEnd('/');

            using (SPSite siteCollection = new SPSite(url))
            {
                if (!disconnect)
                {
                    string portalurl = Params["portalurl"].Value;
                    string portalname = Params["portalname"].Value;
                    siteCollection.PortalUrl = portalurl;
                    siteCollection.PortalName = portalname;
                }
                else
                {
                    siteCollection.PortalUrl = null;
                }
            }
            return 1;
        }

        /// <summary>
        /// Validates the specified key values.
        /// </summary>
        /// <param name="keyValues">The key values.</param>
        public override void Validate(StringDictionary keyValues)
        {
            if (!Params["disconnect"].UserTypedIn)
            {
                Params["portalurl"].IsRequired = true;
                Params["portalname"].IsRequired = true;
            }
            if (Params["disconnect"].UserTypedIn && (Params["portalurl"].UserTypedIn || Params["portalname"].UserTypedIn))
            {
                throw new SPSyntaxException(
                    "The disconnect parameter is incompatible with the protalurl and portalname parameters.");
            }

            base.Validate(keyValues);
        }

        #endregion

    }
}
