using System;
using System.Collections.Specialized;
using System.IO;
using System.Text;
using System.Threading;
using Edfinancial.SharePoint.STSADM.Commands.SPValidators;
using Microsoft.SharePoint;
using Microsoft.SharePoint.Administration;
using Microsoft.SharePoint.Deployment;
using Microsoft.SharePoint.Publishing;

namespace Edfinancial.SharePoint.STSADM.Commands.SiteCollectionSettings
{
    public class ConvertSubSiteToSiteCollection : SPOperation
    {
        /// <summary>
        /// Initializes a new instance of the <see cref="ConvertSubSiteToSiteCollection"/> class.
        /// </summary>
        public ConvertSubSiteToSiteCollection()
        {
            SPParamCollection parameters = new SPParamCollection();
            parameters.Add(new SPParam("sourceurl", "source", true, null, new SPUrlValidator(), "Please specify the source url."));
            parameters.Add(new SPParam("targeturl", "target", true, null, new SPUrlValidator(), "Please specify the target url."));
            parameters.Add(new SPParam("owneremail", "oe", true, null, new SPRegexValidator(@"^[^ \r\t\n\f@]+@[^ \r\t\n\f@]+$"), "Please specify the owner email."));
            parameters.Add(new SPParam("createmanagedpath", "createpath", false, null, null));
            parameters.Add(new SPParam("haltonwarning", "warning", false, null, null));
            parameters.Add(new SPParam("haltonfatalerror", "error", false, null, null));
            parameters.Add(new SPParam("includeusersecurity", "security", false, null, null));
            parameters.Add(new SPParam("exportedfile", "file", false, null, new SPNonEmptyValidator(), "Please specify the exported filename."));
            parameters.Add(new SPParam("nofilecompression", "nofilecompression", false, null, null));
            parameters.Add(new SPParam("ownerlogin", "ol", false, null, new SPNonEmptyValidator(), "Please specify the owner login."));
            parameters.Add(new SPParam("ownername", "on", false, null, new SPNonEmptyValidator(), "Please specify the owner name."));
            parameters.Add(new SPParam("secondarylogin", "sl", false, null, new SPNonEmptyValidator(), "Please specify the secondary owner login."));
            parameters.Add(new SPParam("secondaryname", "sn", false, null, new SPNonEmptyValidator(), "Please specify the secondary owner name."));
            parameters.Add(new SPParam("secondaryemail", "se", false, null, new SPRegexValidator(@"^[^ \r\t\n\f@]+@[^ \r\t\n\f@]+$"), "Please specify the secondary owner email."));
            parameters.Add(new SPParam("lcid", "lcid", false, null, new SPRegexValidator("^[0-9]+$"), "Please specify the language identifier or LCID (example: 1033 for US English)."));
            parameters.Add(new SPParam("sitetemplate", "st", false, null, new SPNonEmptyValidator(), "Please specify the site template."));
            parameters.Add(new SPParam("nositetemplate", "notemplate", false, null, null));
            parameters.Add(new SPParam("donotcreatesite", "donotcreatesite", false, null, null));
            parameters.Add(new SPParam("title", "t", false, null, new SPNonEmptyValidator(), "Please specify the title for the new site."));
            parameters.Add(new SPParam("description", "desc", false, null, new SPNonEmptyValidator(), "Please specify the description for the new site."));
            parameters.Add(new SPParam("hostheaderwebapplicationurl", "hhurl", false, null, new SPUrlValidator(), "Please specify the host header web application url."));
            parameters.Add(new SPParam("quota", "quota", false, null, new SPNonEmptyValidator(), "Please specify the quota template to assign the new site."));
            parameters.Add(new SPParam("deletesource", "deletesource", false, null, null));
            parameters.Add(new SPParam("createsiteinnewdb", "newdb", false, null, null));
            parameters.Add(new SPParam("verbose", "v"));

            SPWebService contentService = SPWebService.ContentService;
            string normalizedDataSource = contentService.DefaultDatabaseInstance.NormalizedDataSource;
            string defaultDatabaseUsername = contentService.DefaultDatabaseUsername;
            string defaultDatabasePassword = contentService.DefaultDatabasePassword;
            if (string.IsNullOrEmpty(normalizedDataSource))
            {
                normalizedDataSource = Environment.MachineName;
            }
            if (defaultDatabaseUsername == null)
            {
                defaultDatabaseUsername = "";
            }
            if (defaultDatabasePassword == null)
            {
                defaultDatabasePassword = "";
            }
            parameters.Add(new SPParam("databaseserver", "ds", false, normalizedDataSource, new SPNonEmptyValidator()));
            parameters.Add(new SPParam("databasename", "dn", false, null, new SPNonEmptyValidator()));
            parameters.Add(new SPParam("databaseuser", "du", false, defaultDatabaseUsername, null));
            parameters.Add(new SPParam("databasepassword", "dp", false, defaultDatabasePassword, null));


            StringBuilder sb = new StringBuilder();
            sb.Append("\r\n\r\nConverts a sub-site to a top level site collection via a managed path.\r\n\r\nParameters:\r\n\t");
            sb.Append("\r\n\t-sourceurl <source location of the existing sub-site or model site collection>");
            sb.Append("\r\n\t-targeturl <target location for the new site collection>");
            sb.Append("\r\n\t-owneremail <someone@example.com>");
            sb.Append("\r\n\t{-sitetemplate <site template if exportedfile is specified - must match the template of the exported site> |");
            sb.Append("\r\n\t -nositetemplate (will automatically apply the sources site template) |");
            sb.Append("\r\n\t -donotcreatesite (specify if the site to import to already exists)}");
            sb.Append("\r\n\t[-createmanagedpath]");
            sb.Append("\r\n\t[-haltonwarning]");
            sb.Append("\r\n\t[-haltonfatalerror]");
            sb.Append("\r\n\t[-includeusersecurity]");
            sb.Append("\r\n\t[-exportedfile <filename of exported site if previously exported>]");
            sb.Append("\r\n\t[-nofilecompression]");
            sb.Append("\r\n\t[-ownerlogin <DOMAIN\\name>]");
            sb.Append("\r\n\t[-ownername <display name>]");
            sb.Append("\r\n\t[-secondaryemail <someone@example.com>]");
            sb.Append("\r\n\t[-secondarylogin <DOMAIN\\name>]");
            sb.Append("\r\n\t[-secondaryname <display name>]");
            sb.Append("\r\n\t[-lcid <language>]");
            sb.Append("\r\n\t[-title <site title>]");
            sb.Append("\r\n\t[-description <site description>]");
            sb.Append("\r\n\t[-hostheaderwebapplicationurl <web application url>]");
            sb.Append("\r\n\t[-quota <quota template>]");
            sb.Append("\r\n\t[-deletesource]");
            sb.Append("\r\n\t[-createsiteinnewdb]");
            sb.Append("\r\n\t[-databaseuser <database username>]");
            sb.Append("\r\n\t[-databasepassword <database password>]");
            sb.Append("\r\n\t[-databaseserver <database server name>]");
            sb.Append("\r\n\t[-databasename <database name>]");
            sb.Append("\r\n\t[-verbose]");

            Init(parameters, sb.ToString());
        }

        #region ISPStsadmCommand Members

        /// <summary>
        /// Gets the help message.
        /// </summary>
        /// <param name="command">The command.</param>
        /// <returns></returns>
        public override string GetHelpMessage(string command)
        {
            return HelpMessage;
        }

        /// <summary>
        /// Runs the specified command.
        /// </summary>
        /// <param name="command">The command.</param>
        /// <param name="keyValues">The key values.</param>
        /// <param name="output">The output.</param>
        /// <returns></returns>
        public override int Run(string command, StringDictionary keyValues, out string output)
        {
            output = string.Empty;
            InitParameters(keyValues);

            SPTriParameterValidator.Validate(
                "nositetemplate", (Params["nositetemplate"].UserTypedIn ? "true" : Params["nositetemplate"].Value),
                "donotcreatesite", (Params["donotcreatesite"].UserTypedIn ? "true" : Params["donotcreatesite"].Value),
                "sitetemplate", Params["sitetemplate"].Value);

            bool verbose = Params["verbose"].UserTypedIn;

            try
            {
                string filename = null;
                if (Params["exportedfile"].UserTypedIn)
                {
                    filename = Params["exportedfile"].Value;
                    if (Params["nofilecompression"].UserTypedIn)
                    {
                        if (string.IsNullOrEmpty(filename) || !Directory.Exists(filename))
                        {
                            throw new SPException(
                                SPResource.GetString("DirectoryNotFoundExceptionMessage", new object[] {filename}));
                        }
                    }
                    else
                    {
                        if (string.IsNullOrEmpty(filename) || !File.Exists(filename))
                        {
                            throw new SPException(
                                SPResource.GetString("FileNotFoundExceptionMessage", new object[] {filename}));
                        }
                    }
                }

                if (Params["createsiteinnewdb"].UserTypedIn && !Params["databasename"].UserTypedIn)
                {
                    throw new SPSyntaxException("databasename is required if creating the site in a new database.");
                }

                string sourceurl = Params["sourceurl"].Value.TrimEnd('/');
                string targeturl = Params["targeturl"].Value.TrimEnd('/');

                using (SPSite sourceSite = new SPSite(sourceurl))
                using (SPWeb sourceWeb = sourceSite.AllWebs[Utilities.GetServerRelUrlFromFullUrl(sourceurl)])
                {
                    if (Params["createmanagedpath"].UserTypedIn)
                    {
                        Console.WriteLine("Adding managed path...");
                        AddManagedPath(targeturl, Params["haltonwarning"].UserTypedIn);
                        Console.WriteLine("Managed path added.\r\n");
                    }

                    if (!Params["exportedfile"].UserTypedIn)
                    {
                        Console.WriteLine("Exporting site...");
                        filename = ExportHelper.ExportSite(sourceurl,
                                              Params["haltonwarning"].UserTypedIn,
                                              Params["haltonfatalerror"].UserTypedIn,
                                              Params["nofilecompression"].UserTypedIn,
                                              true, !verbose, SPIncludeVersions.All, 0, false, true);
                        Console.WriteLine("Site exported.\r\n");
                    }

                    if (!Params["donotcreatesite"].UserTypedIn)
                    {
                        Console.WriteLine("Creating site for import...");
                        CreateSite(sourceurl, targeturl, Params);
                        Console.WriteLine("Site created.\r\n");
                    }

                    using (SPSite targetSite = new SPSite(targeturl))
                    using (SPWeb targetWeb = targetSite.AllWebs[Utilities.GetServerRelUrlFromFullUrl(targeturl)])
                    {
                        if (Params["nositetemplate"].UserTypedIn)
                        {
                            Thread.Sleep(5000); // If you apply the template too quickly an error will sometimes be thrown up.
                            Console.WriteLine("Applying site template based on source...");
                            targetWeb.ApplyWebTemplate(sourceWeb.WebTemplate + "#" + sourceWeb.WebTemplateId);
                            Console.WriteLine("Site template applied.\r\n");

                            Console.WriteLine("Setting required features...");
                            RepairSiteCollectionImportedFromSubSite.AddMissingFeatures(sourceSite, sourceWeb, targetSite,
                                                                                       targetWeb);
                            Console.WriteLine("Required features set.\r\n");
                        }

                        Console.WriteLine("Importing site...");

                        ImportHelper import = new ImportHelper();
                        import.ImportSite(filename,
                                          targeturl,
                                          Params["haltonwarning"].UserTypedIn,
                                          Params["haltonfatalerror"].UserTypedIn,
                                          Params["nofilecompression"].UserTypedIn, true, !verbose, true,
                                          false, SPUpdateVersions.Append);

                        Console.WriteLine("Site imported.\r\n");


                        Console.WriteLine("Repairing imported site...");
                        // Need to add a small delay here as the repair seems to fail occassionally due to a timing issue.
                        Thread.Sleep(10000);
                        int tryCount = 0;
                        while (true)
                        {
                            try
                            {
                                tryCount++;
                                RepairSiteCollectionImportedFromSubSite.RepairSite(sourceurl, targeturl);
                                break;
                            }
                            catch (InvalidPublishingWebException)
                            {
                                if (Params["haltonwarning"].UserTypedIn)
                                    throw;
                                if (tryCount > 3)
                                {
                                    Console.WriteLine("WARNING: Repair of site collection failed - unable to get Pages library.  Manually run 'repairsitecollectionimportedfromsubsite' command to try again.");
                                    break;
                                }
                                else
                                    Thread.Sleep(10000);
                            }
                            catch (OutOfMemoryException)
                            {
                                if (Params["haltonwarning"].UserTypedIn)
                                    throw;

                                Console.WriteLine("WARNING: Server ran out of memory and was not able to complete the repair operation.  Manually run 'repairsitecollectionimportedfromsubsite' command to try again.");
                                break;
                            }
                        }
                        Console.WriteLine("Imported site repaired.\r\n");

                        // Upgrade any redirect links if present.
                        if (sourceSite.WebApplication.Id == targetSite.WebApplication.Id)
                        {
                            Console.WriteLine("Repairing Area upgrade URLs...");
                            // The spsredirect.aspx page which uses the upgrade list only supports server relative urls so
                            // if we change the link to be absolute the redirect won't work so don't bother changing it.
                            // Note that we could get around this if we had a simple redirect page that could take in
                            // a target url - then we could have spsredirect.aspx load the server relative redirect
                            // page which would have the actual url passed into it (unfortunately redirect.aspx does
                            // some funky stuff so we can't use it).
                            UpdateV2ToV3UpgradeAreaUrlMappings.FixUpgradeAreaUrlMappings(sourceSite.WebApplication,
                                                                                         sourceWeb.ServerRelativeUrl,
                                                                                         targetWeb.ServerRelativeUrl);
                            Console.WriteLine("Area upgrade URLs repaired.\r\n");
                        }
                    }

                    if (Params["deletesource"].UserTypedIn)
                    {
                        Console.WriteLine("Deleting source web...");
                        if (!sourceWeb.IsRootWeb)
                        {
                            DeleteSubWebs(sourceWeb.Webs);

                            sourceWeb.Delete();
                            Console.WriteLine("Source web deleted.\r\n");
                            Console.WriteLine("You can find the exported web at " + filename + "\r\n");
                        }
                        else
                            Console.WriteLine("Source web is a root web - cannot delete.");
                    }
                }
            }
            catch (Exception ex)
            {
#if DEBUG
                Console.WriteLine(Utilities.FormatException(ex));
#endif
                throw;
            }

            return 1;
        }

        #endregion

        /// <summary>
        /// Deletes the sub webs.
        /// </summary>
        /// <param name="webs">The webs.</param>
        private static void DeleteSubWebs(SPWebCollection webs)
        {
            foreach (SPWeb web in webs)
            {
                if (web.Webs.Count > 0)
                    DeleteSubWebs(web.Webs);

                web.Delete();
            }
        }

        /// <summary>
        /// Adds the managed path.
        /// </summary>
        /// <param name="targetUrl">The target URL.</param>
        /// <param name="haltOnWarning">if set to <c>true</c> [halt on warning].</param>
        private static void AddManagedPath(string targetUrl, bool haltOnWarning)
        {
            string serverRelUrlFromFullUrl = Utilities.GetServerRelUrlFromFullUrl(targetUrl);
            serverRelUrlFromFullUrl = serverRelUrlFromFullUrl.Trim(new char[] { '/', '*' });

            SPWebApplication webApp = SPWebApplication.Lookup(new Uri(targetUrl));
            SPPrefixCollection prefixCollection = webApp.Prefixes;
            if (prefixCollection.Contains(serverRelUrlFromFullUrl))
            {
                if (haltOnWarning)
                    throw new SPException("Managed path already exists.");
                else 
                    Console.WriteLine("WARNING: Managed path already exists.");
                return;
            }
            prefixCollection.Add(serverRelUrlFromFullUrl, SPPrefixType.ExplicitInclusion);
        }



        /// <summary>
        /// Creates the site.
        /// </summary>
        /// <param name="sourceUrl">The source URL.</param>
        /// <param name="targetUrl">The target URL.</param>
        /// <param name="Params">The params.</param>
        private static void CreateSite(string sourceUrl, string targetUrl, SPParamCollection Params)
        {
            /**
            stsadm.exe -o createsite
            -url <url>
            -owneremail <someone@example.com>
            [-ownerlogin <DOMAIN\name>]
            [-ownername <display name>]
            [-secondaryemail <someone@example.com>]
            [-secondarylogin <DOMAIN\name>]
            [-secondaryname <display name>]
            [-lcid <language>]
            [-sitetemplate <site template>]
            [-title <site title>]
            [-description <site description>]
            [-hostheaderwebapplicationurl <web application url>]
            [-quota <quota template>]
            * */

            /*
            stsadm.exe -o createsiteinnewdb
            -url <url>
            -owneremail <someone@example.com>
            [-ownerlogin <DOMAIN\name>]
            [-ownername <display name>]
            [-secondaryemail <someone@example.com>]
            [-secondarylogin <DOMAIN\name>]
            [-secondaryname <display name>]
            [-lcid <language>]
            [-sitetemplate <site template>]
            [-title <site title>]
            [-description <site description>]
            [-hostheaderwebapplicationurl <web application url>]
            [-quota <quota template>]
            [-databaseuser <database username>]
            [-databasepassword <database password>]
            [-databaseserver <database server name>]
            [-databasename <database name>]
            * */

            StringBuilder args = new StringBuilder();

            if (Params["createsiteinnewdb"].UserTypedIn)
            {
                args.AppendFormat(" -o createsiteinnewdb -url \"{0}\" -owneremail \"{1}\" -databasename \"{2}\"",
                                  targetUrl,
                                  Params["owneremail"].Value,
                                  Params["databasename"].Value);
                
                if (Params["databaseuser"].UserTypedIn)
                    args.AppendFormat(" -databaseuser \"{0}\"", Params["databaseuser"].Value);
                
                if (Params["databasepassword"].UserTypedIn)
                    args.AppendFormat(" -databasepassword \"{0}\"", Params["databasepassword"].Value);
                
                if (Params["databaseserver"].UserTypedIn)
                    args.AppendFormat(" -databaseserver \"{0}\"", Params["databaseserver"].Value);
            }
            else
            {
                args.AppendFormat(" -o createsite -url \"{0}\" -owneremail \"{1}\"",
                                  targetUrl,
                                  Params["owneremail"].Value);
            }
            if (Params["ownerlogin"].UserTypedIn)
                args.AppendFormat(" -ownerlogin \"{0}\"", Params["ownerlogin"].Value);

            if (Params["ownername"].UserTypedIn)
                args.AppendFormat(" -ownername \"{0}\"", Params["ownername"].Value);

            if (Params["secondaryemail"].UserTypedIn)
                args.AppendFormat(" -secondaryemail \"{0}\"", Params["secondaryemail"].Value);

            if (Params["secondarylogin"].UserTypedIn)
                args.AppendFormat(" -secondarylogin \"{0}\"", Params["secondarylogin"].Value);

            if (Params["secondaryname"].UserTypedIn)
                args.AppendFormat(" -secondaryname \"{0}\"", Params["secondaryname"].Value);

            if (Params["lcid"].UserTypedIn)
                args.AppendFormat(" -lcid \"{0}\"", Params["lcid"].Value);

            if (Params["title"].UserTypedIn)
                args.AppendFormat(" -title \"{0}\"", Params["title"].Value);

            if (Params["description"].UserTypedIn)
                args.AppendFormat(" -description \"{0}\"", Params["description"].Value);

            if (Params["hostheaderwebapplicationurl"].UserTypedIn)
                args.AppendFormat(" -hostheaderwebapplicationurl \"{0}\"", Params["hostheaderwebapplicationurl"].Value);

            if (Params["quota"].UserTypedIn)
                args.AppendFormat(" -quota \"{0}\"", Params["quota"].Value);

            if (!Params["nositetemplate"].UserTypedIn)
            {
                string siteTemplate;
                if (Params["sitetemplate"].UserTypedIn)
                    siteTemplate = Params["sitetemplate"].Value;
                else
                {
                    using (SPSite sourceSite = new SPSite(sourceUrl))
                    {
                        using (SPWeb web = sourceSite.AllWebs[Utilities.GetServerRelUrlFromFullUrl(sourceUrl)])
                        {
                            siteTemplate = web.WebTemplate;
                        }
                    }
                }
                args.AppendFormat(" -sitetemplate \"{0}\"", siteTemplate);
            }

            if (Utilities.RunStsAdmOperation(args.ToString(), false) != 0)
                throw new SPException("Error occured creating site.\r\nCOMMAND: " + args);
        }

       
    }
}
