using System.Text;
using System.Xml;
using Edfinancial.SharePoint.STSADM.Commands.SPValidators;
using Microsoft.SharePoint;
using Microsoft.SharePoint.Publishing;

namespace Edfinancial.SharePoint.STSADM.Commands.SiteCollectionSettings
{
    public class CopyNavigation : SPOperation
    {
        public CopyNavigation()
        {
            SPParamCollection parameters = new SPParamCollection();
            parameters.Add(new SPParam("sourceurl", "source", true, null, new SPUrlValidator(), "Please specify the source site collection."));
            parameters.Add(new SPParam("targeturl", "target", true, null, new SPUrlValidator(), "Please specify the target site collection."));
            parameters.Add(new SPParam("globalonly", "g", false, null, null));
            parameters.Add(new SPParam("currentonly", "c", false, null, null));
            parameters.Add(new SPParam("showsubsites", "subsites", false, "true", new SPTrueFalseValidator(), "Please specify either \"true\" or \"false\"."));
            parameters.Add(new SPParam("showpages", "pages", false, "false", new SPTrueFalseValidator(), "Please specify either \"true\" or \"false\"."));
            parameters.Add(new SPParam("deleteexistingglobal", "deleteglobal", false, "true", new SPTrueFalseValidator(), "Please specify either \"true\" or \"false\"."));
            parameters.Add(new SPParam("deleteexistingcurrent", "deletecurrent", false, "true", new SPTrueFalseValidator(), "Please specify either \"true\" or \"false\"."));
            parameters.Add(new SPParam("backuptarget", "backup", false, null, new SPDirectoryExistsAndValidFileNameValidator(), "Make sure the backup file directory exists and a valid filename is provided."));

            StringBuilder sb = new StringBuilder();
            sb.Append("\r\n\r\nCopies the Global and/or Current navigation from one site collection to another.\r\n\r\nParameters:");
            sb.Append("\r\n\t-sourceurl <source site collection url>");
            sb.Append("\r\n\t-targeturl <target site collection url>");
            sb.Append("\r\n\t[-globalonly / -currentonly]");
            sb.Append("\r\n\t[-showsubsites <true (default) | false>]");
            sb.Append("\r\n\t[-showpages <true | false (default)>]");
            sb.Append("\r\n\t[-deleteexistingglobal <true (default) | false>]");
            sb.Append("\r\n\t[-deleteexistingcurrent <true (default) | false>]");
            sb.Append("\r\n\t[-backuptarget <filename>]");

            Init(parameters, sb.ToString());
        }

        #region ISPStsadmCommand Members

        /// <summary>
        /// Gets the help message.
        /// </summary>
        /// <param name="command">The command.</param>
        /// <returns></returns>
        public override string GetHelpMessage(string command)
        {
            return HelpMessage;
        }

        /// <summary>
        /// Runs the specified command.
        /// </summary>
        /// <param name="command">The command.</param>
        /// <param name="keyValues">The key values.</param>
        /// <param name="output">The output.</param>
        /// <returns></returns>
        public override int Run(string command, System.Collections.Specialized.StringDictionary keyValues, out string output)
        {
            output = string.Empty;

            InitParameters(keyValues);

            SPBinaryParameterValidator.Validate("globalonly", (Params["globalonly"].UserTypedIn ? "true" : Params["globalonly"].Value),
                "currentonly", (Params["currentonly"].UserTypedIn ? "true" : Params["currentonly"].Value));


            string sourceurl = Params["sourceurl"].Value.TrimEnd('/');
            string targeturl = Params["targeturl"].Value.TrimEnd('/');
            bool globalOnly = Params["globalonly"].UserTypedIn;
            bool currentOnly = Params["currentonly"].UserTypedIn;
            bool showSubSites = bool.Parse(Params["showsubsites"].Value);
            bool showPages = bool.Parse(Params["showpages"].Value);
            bool deleteExistingGlobal = bool.Parse(Params["deleteexistingglobal"].Value);
            bool deleteExistingCurrent = bool.Parse(Params["deleteexistingcurrent"].Value);

            XmlDocument xmlDoc;
            // Get the XML from our source site.
            using (SPSite sourceSite = new SPSite(sourceurl))
            using (SPWeb sourceWeb = sourceSite.AllWebs[Utilities.GetServerRelUrlFromFullUrl(sourceurl)])
            {
                PublishingWeb sourcePubWeb = PublishingWeb.GetPublishingWeb(sourceWeb);
                xmlDoc = EnumNavigation.GetNavigationAsXml(sourcePubWeb);
            }

            // If the user only wants to copy the global or current then remove the other element
            if (currentOnly)
                xmlDoc.SelectSingleNode("//Navigation/Global").RemoveAll();
            if (globalOnly)
                xmlDoc.SelectSingleNode("//Navigation/Current").RemoveAll();

            // Set the navigation using the XML we got from the source.
            using (SPSite targetSite = new SPSite(targeturl))
            using (SPWeb targetWeb = targetSite.AllWebs[Utilities.GetServerRelUrlFromFullUrl(targeturl)])
            {
                if (keyValues.ContainsKey("backuptarget"))
                {
                    PublishingWeb targetPubWeb = PublishingWeb.GetPublishingWeb(targetWeb);
                    XmlDocument xmlBackupDoc = EnumNavigation.GetNavigationAsXml(targetPubWeb);
                    xmlBackupDoc.Save(keyValues["backuptarget"]);
                }
                SetNavigationNodes.SetNavigation(targetSite, targetWeb, xmlDoc, showSubSites, showPages, deleteExistingGlobal, deleteExistingCurrent);
            }

            return 1;
        }

        #endregion
    }
}
