using System.Collections.Specialized;
using Edfinancial.SharePoint.STSADM.Commands.SPValidators;
using Microsoft.SharePoint;

namespace Edfinancial.SharePoint.STSADM.Commands.SiteCollectionSettings
{
    public class DeleteWeb2 : SPOperation
    {
        /// <summary>
        /// Initializes a new instance of the <see cref="DeleteWeb2"/> class.
        /// </summary>
        public DeleteWeb2()
        {
            SPParamCollection parameters = new SPParamCollection();
            parameters.Add(new SPParam("url", "url", true, null, new SPUrlValidator(), "Please specify the URL of the web to delete."));
            parameters.Add(new SPParam("recurse", "r", false, null, null));
            Init(parameters, "\r\n\r\nDeletes a web.\r\n\r\nParameters:\r\n\t-url <url of web to delete>\r\n\t[-recurse (delete all sub-sites)]");
        }

        #region ISPStsadmCommand Members

        /// <summary>
        /// Gets the help message.
        /// </summary>
        /// <param name="command">The command.</param>
        /// <returns></returns>
        public override string GetHelpMessage(string command)
        {
            return HelpMessage;
        }

        /// <summary>
        /// Runs the specified command.
        /// </summary>
        /// <param name="command">The command.</param>
        /// <param name="keyValues">The key values.</param>
        /// <param name="output">The output.</param>
        /// <returns></returns>
        public override int Run(string command, StringDictionary keyValues, out string output)
        {
            output = string.Empty;

            InitParameters(keyValues);

            string url = Params["url"].Value.TrimEnd('/');
            
            using (SPSite site = new SPSite(url))
            using (SPWeb web = site.AllWebs[Utilities.GetServerRelUrlFromFullUrl(url)])
            {
                if (web.IsRootWeb)
                {
                    throw new SPException(SPResource.GetString("CannotDelRootwebByDelweb", new object[0]));
                }

                if (web.Webs.Count > 0 && !Params["recurse"].UserTypedIn)
                {
                    throw new SPException(
                        string.Format(
                            "Error deleting Web site \"{0}\".  You can't delete a site that has subsites unless you pass the \"recurse\" flag into the command.",
                            web.ServerRelativeUrl));
                }

                if (web.Webs.Count > 0 && Params["recurse"].UserTypedIn)
                    DeleteSubWebs(web.Webs);

                web.Delete();

            }

            return 1;
        }

        #endregion

        /// <summary>
        /// Deletes the sub webs.
        /// </summary>
        /// <param name="webs">The webs.</param>
        private static void DeleteSubWebs(SPWebCollection webs)
        {
            foreach (SPWeb web in webs)
            {
                if (web.Webs.Count > 0)
                    DeleteSubWebs(web.Webs);

                web.Delete();
            }
        }

    }
}
