using System.Text;
using Edfinancial.SharePoint.STSADM.Commands.SPValidators;
using Microsoft.SharePoint.Deployment;

namespace Edfinancial.SharePoint.STSADM.Commands.SiteCollectionSettings
{
    public class Import2 : SPOperation
    {

        /// <summary>
        /// Initializes a new instance of the <see cref="Import2"/> class.
        /// </summary>
        public Import2()
        {
            SPParamCollection parameters = new SPParamCollection();
            parameters.Add(new SPParam("url", "url", true, null, new SPUrlValidator()));
            parameters.Add(new SPParam("filename", "f", true, null, new SPNonEmptyValidator()));
            parameters.Add(new SPParam("quiet", "quiet", false, null, null));
            parameters.Add(new SPParam("includeusersecurity", "security", false, null, null));
            parameters.Add(new SPParam("haltonwarning", "warning", false, null, null));
            parameters.Add(new SPParam("haltonfatalerror", "error", false, null, null));
            parameters.Add(new SPParam("nologfile", "nolog", false, null, null));
            parameters.Add(new SPParam("updateversions", "updatev", false, SPUpdateVersions.Append.ToString(), new SPIntRangeValidator(1, 3)));
            parameters.Add(new SPParam("nofilecompression", "nofilecompression", false, null, null));
            parameters.Add(new SPParam("retainobjectidentity", "retainid", false, null, null));
            
            StringBuilder sb = new StringBuilder();
            sb.Append("\r\n\r\nAn improved version of the built-in \"import\" command.  Allows the ability to retain the original object identifiers.\r\n\r\nParameters:\r\n");
            sb.Append("\t-url <URL to import to (parent if retainobjectidentity, otherwise full url of target location)>\r\n");
            sb.Append("\t-filename <import file name>\r\n");
            sb.Append("\t[-includeusersecurity]\r\n");
            sb.Append("\t[-haltonwarning]\r\n");
            sb.Append("\t[-haltonfatalerror]\r\n");
            sb.Append("\t[-nologfile]\r\n");
            sb.Append("\t[-updateversions <1-3>\r\n");
            sb.Append("\t    1 - Add new versions to the current file (default)\r\n");
            sb.Append("\t    2 - Overwrite the file and all its versions (delete then insert)\r\n");
            sb.Append("\t    3 - Ignore the file if it exists on the destination]\r\n");
            sb.Append("\t[-nofilecompression]\r\n");
            sb.Append("\t[-quiet]\r\n");
            sb.Append("\t[-retainobjectidentity (the source must not exist in the content database and the url parameter must refer to the parent web)]\r\n");

            Init(parameters, sb.ToString());
        }

        #region ISPStsadmCommand Members

        /// <summary>
        /// Gets the help message.
        /// </summary>
        /// <param name="command">The command.</param>
        /// <returns></returns>
        public override string GetHelpMessage(string command)
        {
            return HelpMessage;
        }

        /// <summary>
        /// Runs the specified command.
        /// </summary>
        /// <param name="command">The command.</param>
        /// <param name="keyValues">The key values.</param>
        /// <param name="output">The output.</param>
        /// <returns></returns>
        public override int Run(string command, System.Collections.Specialized.StringDictionary keyValues, out string output)
        {
            output = string.Empty;

            InitParameters(keyValues);

            string url = Params["url"].Value;
            string filename = Params["filename"].Value;
            bool includeUserSecurity = Params["includeusersecurity"].UserTypedIn;
            bool haltOnWarning = Params["haltonwarning"].UserTypedIn;
            bool haltOnFatalError = Params["haltonfatalerror"].UserTypedIn;
            bool logFile = !Params["nologfile"].UserTypedIn;
            bool noFileCompression = Params["nofilecompression"].UserTypedIn;
            bool quiet = Params["quiet"].UserTypedIn;
            SPUpdateVersions updateVersions = SPUpdateVersions.Append;
            if (Params["updateversions"].UserTypedIn)
                updateVersions = (SPUpdateVersions) int.Parse(Params["updateversions"].Value);
            bool retainObjectIdentity = Params["retainobjectidentity"].UserTypedIn;

            ImportHelper import = new ImportHelper();

            import.ImportSite(
                filename,
                url,
                haltOnWarning,
                haltOnFatalError,
                noFileCompression,
                includeUserSecurity,
                quiet,
                logFile,
                retainObjectIdentity,
                updateVersions);

            return 1;
        }

        #endregion

    }
}
