using System;
using System.Collections.Specialized;
using System.IO;
using System.Text;
using Edfinancial.SharePoint.STSADM.Commands.SPValidators;
using Microsoft.SharePoint;

namespace Edfinancial.SharePoint.STSADM.Commands.SiteCollectionSettings
{
    public class MoveSite : SPOperation
    {
        /// <summary>
        /// Initializes a new instance of the <see cref="MoveSite"/> class.
        /// </summary>
        public MoveSite()
        {
            SPParamCollection parameters = new SPParamCollection();
            parameters.Add(new SPParam("url", "url", true, null, new SPUrlValidator()));
            parameters.Add(new SPParam("moveto", "move", true, null, new SPUrlValidator()));

            StringBuilder sb = new StringBuilder();
            sb.Append("\r\n\r\nMoves a site collection (wraps the backup, deletesite, and restore operations into one operation).\r\n\r\nParameters:");
            sb.Append("\r\n\t-url <url>");
            sb.Append("\r\n\t-moveto <url of a new target path to move the upgraded site collection to>");

            Init(parameters, sb.ToString());
        }

        #region ISPStsadmCommand Members

        /// <summary>
        /// Gets the help message.
        /// </summary>
        /// <param name="command">The command.</param>
        /// <returns></returns>
        public override string GetHelpMessage(string command)
        {
            return HelpMessage;
        }

        /// <summary>
        /// Runs the specified command.
        /// </summary>
        /// <param name="command">The command.</param>
        /// <param name="keyValues">The key values.</param>
        /// <param name="output">The output.</param>
        /// <returns></returns>
        public override int Run(string command, StringDictionary keyValues, out string output)
        {
            output = string.Empty;

            InitParameters(keyValues);


            MoveSiteCollection(Params["url"].Value, Params["moveto"].Value);

            return 1;
        }
        
        #endregion

        /// <summary>
        /// Moves the site collection.
        /// </summary>
        /// <param name="existingUrl">The existing URL.</param>
        /// <param name="newUrl">The new URL.</param>
        internal static void MoveSiteCollection(string existingUrl, string newUrl)
        {
            /*stsadm.exe -o backup
                -url <url>
                -filename <filename>
                [-overwrite]*/
            string filename = Path.Combine(Path.GetTempPath(), Guid.NewGuid() + ".bak");

            string cmd = string.Format(" -o backup -url \"{0}\" -filename \"{1}\"", existingUrl, filename);

            if (Utilities.RunStsAdmOperation(cmd.ToString(), true) != 0)
                throw new SPException("Error occured backing up site for move.\r\nCOMMAND: stsadm.exe" + cmd);

            /*stsadm.exe -o deletesite
               -url <url>
               -deleteadaccounts <true/false>*/
            cmd = string.Format(" -o deletesite -url \"{0}\"", existingUrl);

            if (Utilities.RunStsAdmOperation(cmd.ToString(), true) != 0)
                throw new SPException("Error occured deleting source site for move.\r\nCOMMAND: stsadm.exe" + cmd);

            /*stsadm.exe -o restore
                -url <url>
                -filename <filename>
                [-hostheaderwebapplicationurl <web application url>]
                [-overwrite]*/

            cmd = string.Format(" -o restore -url \"{0}\" -filename \"{1}\"", newUrl, filename);

            try
            {
                if (Utilities.RunStsAdmOperation(cmd.ToString(), true) != 0)
                    throw new SPException("Error occured restoring site for move.\r\nCOMMAND: stsadm.exe" + cmd);
            }
            catch (SPException)
            {
                Console.WriteLine("\r\nSite backup file can be located here: {0}", filename);
                throw;
            }
            File.Delete(filename);
        }
    }
}
