using System;
using System.Text;
using Edfinancial.SharePoint.STSADM.Commands.SPValidators;
using Microsoft.SharePoint.Administration;
using Microsoft.SharePoint.Portal.SiteDirectory;
using System.Reflection;

namespace Edfinancial.SharePoint.STSADM.Commands.SiteDirectory
{
    public class SetSiteDirectoryScanViewUrls : SPOperation
    {
        /// <summary>
        /// Initializes a new instance of the <see cref="SetSiteDirectoryScanViewUrls"/> class.
        /// </summary>
        public SetSiteDirectoryScanViewUrls()
        {
            SPParamCollection parameters = new SPParamCollection();
            parameters.Add(new SPParam("urls", "urls", true, null, new SPNonEmptyValidator(), "Enter URLs to site directory views that you want to scan for broken links. Separate multiple views by commas. Example: http://[ServerName]/sitedirectory/siteslist/allitems.aspx."));
            parameters.Add(new SPParam("updatesiteproperties", "update", true, null, new SPTrueFalseValidator(), "Please specify whether you would like to update the title and description of links in the site directory automatically."));

            StringBuilder sb = new StringBuilder();
            sb.Append("\r\n\r\nSets the site directory links scan job options.\r\n\r\nParameters:");
            sb.Append("\r\n\t-urls <views to scan (separate multiple views with a comma)>");
            sb.Append("\r\n\t-updatesiteproperties <true | false>");
            Init(parameters, sb.ToString());
        }

        #region ISPStsadmCommand Members

        /// <summary>
        /// Gets the help message.
        /// </summary>
        /// <param name="command">The command.</param>
        /// <returns></returns>
        public override string GetHelpMessage(string command)
        {
            return HelpMessage;
        }

        /// <summary>
        /// Runs the specified command.
        /// </summary>
        /// <param name="command">The command.</param>
        /// <param name="keyValues">The key values.</param>
        /// <param name="output">The output.</param>
        /// <returns></returns>
        public override int Run(string command, System.Collections.Specialized.StringDictionary keyValues, out string output)
        {
            output = string.Empty;

            InitParameters(keyValues);

            string urls = Params["urls"].Value;
            bool updateSiteProperties = bool.Parse(Params["updatesiteproperties"].Value);


            SPTimerService timerService = SPFarm.Local.TimerService;
            if (null == timerService)
            {
                throw new LinksCheckerException("Links Checker Timer Service Not Found.");
            }

            LinksCheckerJob job = timerService.JobDefinitions.GetValue<LinksCheckerJob>("SPS-SiteDirectoryLinksChecker");
            if (null == job)
            {
                // As I've grown tired of trying to get around the limitations of working with only public accessors I've decided to call the 
                // internal accessors so this code now works exactly as the LinksCheckerJobSettings.SaveViewUrls() method which is what is called
                // when you adjust the settings via the browser.

                //job = new LinksCheckerJob(timerService);
                ConstructorInfo nonPublicConstructorInfo =
                    typeof (LinksCheckerJob).GetConstructor(
                        BindingFlags.NonPublic | BindingFlags.Instance | BindingFlags.InvokeMethod, null,
                        new Type[] {typeof (SPTimerService)}, null);

                job = (LinksCheckerJob)nonPublicConstructorInfo.Invoke(new object[] { timerService });


                //job.SetDefaults();
                MethodInfo setDefaults =
                    typeof (LinksCheckerJob).GetMethod("SetDefaults",
                                                       BindingFlags.NonPublic | BindingFlags.Instance |
                                                       BindingFlags.InvokeMethod);
                setDefaults.Invoke(job, new object[] {});
            }
            job.SiteDirectoryConfig = urls;
            job.IsMetaDataSyncEnabled = updateSiteProperties;
            job.Update();

            return 1;
        }

        #endregion
    }
}
