using System;
using System.Collections.Generic;
using System.Collections.Specialized;
using Microsoft.SharePoint;
using Microsoft.SharePoint.Administration;
using Microsoft.SharePoint.StsAdmin;

namespace Edfinancial.SharePoint.STSADM.Commands.TimerJob
{
    /// <summary>
    /// Displays information about a Timer Job Definition in a site collection.
    /// </summary>
    public class GetJobInfo : ISPStsadmCommand
    {
        /// <summary>
        /// Gets the help message.
        /// </summary>
        /// <param name="command">The command.</param>
        /// <returns></returns>
        public string GetHelpMessage(string command)
        {
            return "\r\n\r\nDisplays Information about the specified Timer Job\r\n\r\nParameters:\r\n\t-job <job name>\r\n\t-url <url of site collection>";
        }

        /// <summary>
        /// Runs the specified command.
        /// </summary>
        /// <param name="command">The command.</param>
        /// <param name="keyValues">The key values.</param>
        /// <param name="output">The output.</param>
        /// <returns></returns>
        public int Run(string command, StringDictionary keyValues, out string output)
        {
            output = string.Empty;

            #region Check Arguments
            if (!keyValues.ContainsKey("job") || keyValues["job"].Length == 0)
            {
                output = "Please specify the job name.";
                output += GetHelpMessage(command);
                return 0;
            }

            if (!keyValues.ContainsKey("url") || keyValues["url"].Length == 0)
            {
                output = "Please specify a url of a site collection.";
                output += GetHelpMessage(command);
                return 0;
            }
            #endregion

            string specifiedJobName = keyValues["job"];
            string siteCollectionUrl = keyValues["url"];


            using (SPSite site = new SPSite(siteCollectionUrl))
            {
                List<SPJobDefinition> jobs = Helper.GetJobDefinition(site, specifiedJobName);

                if (jobs != null && jobs.Count > 1)
                {
                    throw new ApplicationException("More than one job found matching the specified name " +
                                                   specifiedJobName);
                }

                if (jobs != null && jobs.Count > 0)
                {
                    SPJobDefinition job = jobs[0];
                    output += "\r\nFound Job, showing information:\r\n";
                    output += Helper.GetJobInformation(job);
                }
                else
                {
                    throw new ApplicationException("Could not find a job with name [" + specifiedJobName + "] for that site collection");
                }
            }

            return 1;
        }

        
    }
}
