using System.Collections.Specialized;
using Microsoft.SharePoint;
using Microsoft.SharePoint.Administration;
using Microsoft.SharePoint.StsAdmin;

namespace Edfinancial.SharePoint.STSADM.Commands.TimerJob
{
    /// <summary>
    /// Displays information about all Timer Job Definitions in a site collection.
    /// </summary>
    public class GetJobInfos : ISPStsadmCommand
    {
        public string GetHelpMessage(string command)
        {
            return "\r\n\r\nDisplays Information about the all Timer Jobs in the given site collection\r\n\r\nParemeters:\r\n\t-url <url of site collection>";
        }

        public int Run(string command, StringDictionary keyValues, out string output)
        {
            output = string.Empty;

            #region Check Arguments
            if (!keyValues.ContainsKey("url") || keyValues["url"].Length == 0)
            {
                output = "Please specify a url of a site collection.";
                output += GetHelpMessage(command);
                return 0;
            }
            #endregion

            string siteCollectionUrl = keyValues["url"];

            using (SPSite site = new SPSite(siteCollectionUrl))
            {
                foreach (SPJobDefinition job in site.WebApplication.JobDefinitions)
                {
                    if (job != null)
                    {
                        output += Helper.GetJobInformation(job);
                        output += "\r\n";
                    }
                }
            }

            return 1;
        }
    }
}
