using System;
using System.Collections.Generic;
using System.Collections.Specialized;
using Microsoft.SharePoint;
using Microsoft.SharePoint.Administration;
using Microsoft.SharePoint.StsAdmin;

namespace Edfinancial.SharePoint.STSADM.Commands.TimerJob
{
    /// <summary>
    /// Sharepoint STSADM Command to start a specific timer job NOW.
    /// </summary>
    public class RunTimerJob : ISPStsadmCommand
    {
        /// <summary>
        /// Gets the help message.
        /// </summary>
        /// <param name="command">The command.</param>
        /// <returns></returns>
        public string GetHelpMessage(string command)
        {
            return "\r\n\r\nStarts the specified Timer Job\r\n\r\nParameters:\r\n\t-job <job name>\r\n\t[-url <url of site collection>]";
        }

        /// <summary>
        /// Runs the specified command.
        /// </summary>
        /// <param name="command">The command.</param>
        /// <param name="keyValues">The key values.</param>
        /// <param name="output">The output.</param>
        /// <returns></returns>
        public int Run(string command, StringDictionary keyValues, out string output)
        {
            output = string.Empty;

            if (!keyValues.ContainsKey("job") || keyValues["job"].Length==0)
            {
                output = "Please specify the job name.";
                output += GetHelpMessage(command);
                return 0;
            }

            if(keyValues.ContainsKey("url") && keyValues["url"].Length==0)
            {
                output = "Please specify an url of a site collection.";
                output += GetHelpMessage(command);
                return 0;
            }

            string specifiedJobName = keyValues["job"];
            string siteCollectionUrl = keyValues["url"];
            Guid databaseId;

            List<SPJobDefinition> jobs;
            if (!string.IsNullOrEmpty(siteCollectionUrl))
            {
                using (SPSite site = new SPSite(siteCollectionUrl))
                {
                    jobs = Helper.GetJobDefinition(site, specifiedJobName);
                    databaseId = site.ContentDatabase.Id;
                }
            }
            else
                jobs = Helper.GetJobDefinition(specifiedJobName, out databaseId);

            if (jobs != null && jobs.Count > 0)
            {
                output += "Executing job " + specifiedJobName + " for Content Database " + databaseId;
                try
                {
                    foreach (SPJobDefinition job in jobs)
                        job.Execute(databaseId);
                }
                catch (Exception ex)
                {
                    output += "\r\nException occured:\r\n" + ex;
                }
            }
            else
            {
                output += "\r\nCould not find a job with name [" + specifiedJobName + "] for that site collection";
            }
            
            return 1;
        }
    }
}
