using System.Collections.Specialized;
using System.Data.SqlClient;
using System.Text;
using System.Xml;
using Edfinancial.SharePoint.STSADM.Commands.SPValidators;
using Edfinancial.SharePoint.STSADM.Commands.UserProfiles.UserProfileProperties2003TableAdapters;
using Microsoft.Office.Server;
using Microsoft.Office.Server.UserProfiles;

namespace Edfinancial.SharePoint.STSADM.Commands.UserProfiles
{
    public class Gen2003To2007ProfilePropertyMap : SPOperation
    {
        /// <summary>
        /// Initializes a new instance of the <see cref="Gen2003To2007ProfilePropertyMap"/> class.
        /// </summary>
        public Gen2003To2007ProfilePropertyMap()
        {
            SPParamCollection parameters = new SPParamCollection();
            parameters.Add(new SPParam("profdb2003", "server", true, null, new SPNonEmptyValidator(), "Please specify the Profiles Database Server for SharePoint 2003 (profdb2003)."));
            parameters.Add(new SPParam("profdbname2003", "dbname", true, null, new SPNonEmptyValidator(), "Please specify the Profiles Database Name for SharePoint 2003 (profdbname2003)."));
            parameters.Add(new SPParam("sspname", "ssp", true, null, new SPNonEmptyValidator(), "Please specify the SSP name."));
            parameters.Add(new SPParam("output", "o", true, null, new SPDirectoryExistsAndValidFileNameValidator(), "Please specify the output file name (i.e., \"c:\\propertymap.xml\")."));

            StringBuilder sb = new StringBuilder();
            sb.Append("\r\n\r\nGenerates a mapping file to map SharePoint 2003 profile data into a SharePoint 2007 SSP.\r\n\r\nParameters:");
            sb.Append("\r\n\t-profdb2003 <database server containing the 2003 profile database>");
            sb.Append("\r\n\t-profdbname2003 <2003 profile database name>");
            sb.Append("\r\n\t-sspname <name of the 2007 SSP>");
            sb.Append("\r\n\t-output <mapping file to generate>");
            Init(parameters, sb.ToString());
        }

        #region ISPStsadmCommand Members

        /// <summary>
        /// Gets the help message.
        /// </summary>
        /// <param name="command">The command.</param>
        /// <returns></returns>
        public override string GetHelpMessage(string command)
        {
            return HelpMessage;
        }

        /// <summary>
        /// Runs the specified command.
        /// </summary>
        /// <param name="command">The command.</param>
        /// <param name="keyValues">The key values.</param>
        /// <param name="output">The output.</param>
        /// <returns></returns>
        public override int Run(string command, StringDictionary keyValues, out string output)
        {
            output = string.Empty;

            InitParameters(keyValues);

            string profdb = Params["profdb2003"].Value;
            string profdbname = Params["profdbname2003"].Value;
            string sspname = Params["sspname"].Value;
            string outputFileName = Params["output"].Value;

            UserProfileConfigManager prof = new UserProfileConfigManager(ServerContext.GetContext(sspname));

            PropertyCollection propColl = prof.GetProperties();

            XmlDocument map = new XmlDocument();
            map.AppendChild(map.CreateElement("ProfileMap"));
            XmlElement propertiesNode = map.CreateElement("Properties-2007");
            map.DocumentElement.AppendChild(propertiesNode);
            propertiesNode.AppendChild(map.CreateComment("The following property list is here for reference only and can be safely deleted."));

            foreach (Property prop in propColl)
            {
                if (prop.IsSection)
                    continue;

                XmlElement propXml = map.CreateElement("Property");
                propXml.SetAttribute("PropertyName", prop.Name);
                propXml.SetAttribute("DisplayName", prop.DisplayName);
                propXml.SetAttribute("ManagedPropertyName", prop.ManagedPropertyName);
                propXml.SetAttribute("Description", prop.Description);
                propXml.SetAttribute("Length", prop.Length.ToString());
                propXml.SetAttribute("Type", prop.Type);
                propXml.SetAttribute("IsAlias", prop.IsAlias.ToString());
                propXml.SetAttribute("IsMultivalued", prop.IsMultivalued.ToString());
                propXml.SetAttribute("IsUserEditable", prop.IsUserEditable.ToString());
                propXml.SetAttribute("IsVisibleOnEditor", prop.IsVisibleOnEditor.ToString());
                propXml.SetAttribute("IsVisibleOnViewer", prop.IsVisibleOnViewer.ToString());

                propertiesNode.AppendChild(propXml);

            }


            PropertyListTableAdapter adapter = new PropertyListTableAdapter();
            adapter.Connection = new SqlConnection("Data Source=" + profdb + ";Initial Catalog=" + profdbname + ";Integrated Security=True");

            UserProfileProperties2003.PropertyListDataTable table = adapter.GetData();


            foreach (UserProfileProperties2003.PropertyListRow row in table.Rows)
            {
                XmlElement propXml = map.CreateElement("Property");
                propXml.SetAttribute("PropertyID", row.PropertyID.ToString());
                propXml.SetAttribute("PropertyName", row.PropertyName);
                propXml.SetAttribute("IsMultiValue", row.IsMultiValue.ToString());
                if (!row.IsLengthNull())
                    propXml.SetAttribute("Length", row.Length.ToString());

                Property prop = propColl.GetPropertyByName(row.PropertyName);
                if (prop == null)
                {
                    prop = propColl.GetPropertyByName("SPS-" + row.PropertyName);
                }
                string mappedPropertyName = "";

                if (prop != null)
                {
                    mappedPropertyName = prop.Name;

                }
                else
                {
                    if (!row.IsDisplayNameNull())
                        propXml.SetAttribute("DisplayName", row.DisplayName);

                    propXml.SetAttribute("DataTypeName", row.DataTypeName);
                    propXml.SetAttribute("IsAlias", row.IsAlias.ToString());
                    propXml.SetAttribute("IsEditable", row.IsEditable.ToString());
                    propXml.SetAttribute("IsPrivate", row.IsPrivate.ToString());
                    propXml.SetAttribute("IsVisibleOnEditor", "True");
                    propXml.SetAttribute("IsVisibleOnViewer", "True");
                }

                propXml.SetAttribute("propertyName-2007", mappedPropertyName);

                map.DocumentElement.AppendChild(propXml);
            }

            map.Save(outputFileName);

            return 1;
        }

        #endregion
    }
}
