using System.Collections.Specialized;
using System.Text;
using Edfinancial.SharePoint.STSADM.Commands.SPValidators;
using Microsoft.Office.Server;
using Microsoft.Office.Server.UserProfiles;

namespace Edfinancial.SharePoint.STSADM.Commands.UserProfiles
{
    public class SetPictureUrlNewPath : SPOperation
    {
        /// <summary>
        /// Initializes a new instance of the <see cref="SetPictureUrlNewPath"/> class.
        /// </summary>
        public SetPictureUrlNewPath()
        {
            SPParamCollection parameters = new SPParamCollection();
            parameters.Add(new SPParam("librarypath", "l", true, null, new SPNonEmptyValidator(), "Please specify the picture library path."));
            parameters.Add(new SPParam("sspname", "ssp", true, null, new SPNonEmptyValidator(), "Please specify the SSP name."));
            parameters.Add(new SPParam("username", "u", false, null, new SPNonEmptyValidator(), "Please specify the username."));

            StringBuilder sb = new StringBuilder();
            sb.Append("\r\n\r\nFixes the pathing for all user photos or a single user's photo as the result of moving a picture library (assumes file names have not changed).\r\n\r\nParameters:");
            sb.Append("\r\n\t-sspname <name of the 2007 SSP>");
            sb.Append("\r\n\t-librarypath <path to new photo library (i.e., \"http://intranet/hr/EmployeePictures/\")>");
            sb.Append("\r\n\t[-username <DOMAIN\\name>]");

            Init(parameters, sb.ToString());
        }
        #region ISPStsadmCommand Members

        /// <summary>
        /// Gets the help message.
        /// </summary>
        /// <param name="command">The command.</param>
        /// <returns></returns>
        public override string GetHelpMessage(string command)
        {
            return HelpMessage;
        }

        /// <summary>
        /// Runs the specified command.
        /// </summary>
        /// <param name="command">The command.</param>
        /// <param name="keyValues">The key values.</param>
        /// <param name="output">The output.</param>
        /// <returns></returns>
        public override int Run(string command, StringDictionary keyValues, out string output)
        {
            output = string.Empty;

            InitParameters(keyValues);

            string username = null;

            if (Params["username"].UserTypedIn)
                username = Params["username"].Value;
            string sspname = Params["sspname"].Value;
            string librarypath = Params["librarypath"].Value;

            UserProfileManager profManager = new UserProfileManager(ServerContext.GetContext(sspname));
            
            if (!string.IsNullOrEmpty(username))
            {
                if (!profManager.UserExists(username))
                {
                    output += "The username specified cannot be found.";
                    return 0;
                }
                UserProfile profile = profManager.GetUserProfile(username);
                if (SetPicture(profile["PictureURL"], librarypath))
                    profile.Commit();
            }
            else
            {
                foreach (UserProfile profile in profManager)
                {
                    if (SetPicture(profile["PictureURL"], librarypath))
                        profile.Commit();
                }
            }


            return 1;
        }

        #endregion

        /// <summary>
        /// Sets the picture.
        /// </summary>
        /// <param name="val">The val.</param>
        /// <param name="libraryPath">The library path.</param>
        /// <returns></returns>
        private static bool SetPicture(UserProfileValueCollection val, string libraryPath)
        {
            string currentUrl;
            if (val.Value == null)
                return false;
            else
                currentUrl = val.Value.ToString();

            if (string.IsNullOrEmpty(currentUrl))
                return false;

            string newUrl = libraryPath.TrimEnd('/') + "/";
            if (currentUrl.IndexOf('/') < 0)
                newUrl += currentUrl.Trim();
            else
            {
                string file = currentUrl.Substring(currentUrl.LastIndexOf('/'), currentUrl.Length - currentUrl.LastIndexOf('/'));
                newUrl += file.Trim('/');
            }
            val.Value = newUrl;

            return true;
        }
    }
}
