using System.Collections.Specialized;
using System.Text;
using Edfinancial.SharePoint.STSADM.Commands.SPValidators;
using Microsoft.Office.Server;
using Microsoft.Office.Server.UserProfiles;

namespace Edfinancial.SharePoint.STSADM.Commands.UserProfiles
{
    public class SetUserProfileDefaultAccessAccount : SPOperation
    {
        /// <summary>
        /// Initializes a new instance of the <see cref="SetUserProfileDefaultAccessAccount"/> class.
        /// </summary>
        public SetUserProfileDefaultAccessAccount()
        {
            SPParamCollection parameters = new SPParamCollection();
            parameters.Add(new SPParam("username", "u", true, null, new SPNonEmptyValidator(), "Please specify the username."));
            parameters.Add(new SPParam("password", "p", true, null, new SPNonEmptyValidator(), "Please specify the password."));
            parameters.Add(new SPParam("sspname", "ssp", true, null, new SPNonEmptyValidator(), "Please specify the SSP name."));

            StringBuilder sb = new StringBuilder();
            sb.Append("\r\n\r\nSets the default access account for user profiles.\r\n\r\nParameters:");
            sb.Append("\r\n\t-username <DOMAIN\\name>");
            sb.Append("\r\n\t-password <password>");
            sb.Append("\r\n\t-sspname <name of the SSP>");

            Init(parameters, sb.ToString());
        }
        #region ISPStsadmCommand Members

        /// <summary>
        /// Gets the help message.
        /// </summary>
        /// <param name="command">The command.</param>
        /// <returns></returns>
        public override string GetHelpMessage(string command)
        {
            return HelpMessage;
        }

        /// <summary>
        /// Runs the specified command.
        /// </summary>
        /// <param name="command">The command.</param>
        /// <param name="keyValues">The key values.</param>
        /// <param name="output">The output.</param>
        /// <returns></returns>
        public override int Run(string command, StringDictionary keyValues, out string output)
        {
            output = string.Empty;

            InitParameters(keyValues);

            string username = Params["username"].Value;
            string password = Params["password"].Value;
            string sspname = Params["sspname"].Value;

            UserProfileConfigManager prof = new UserProfileConfigManager(ServerContext.GetContext(sspname));
            
            DataSource dataSource = prof.GetDataSource();

            dataSource.SetDefaultImportAccount(username, password);

            return 1;
        }

        #endregion
    }
}
