using System;
using System.Collections.Specialized;
using System.Text;
using Edfinancial.SharePoint.STSADM.Commands.SPValidators;
using Microsoft.SharePoint.Administration;

namespace Edfinancial.SharePoint.STSADM.Commands.WebParts
{
    public class SetAllowAccessToOnlineGallery : SPOperation
    {
        /// <summary>
        /// Initializes a new instance of the <see cref="SetAllowAccessToOnlineGallery"/> class.
        /// </summary>
        public SetAllowAccessToOnlineGallery()
        {
            SPParamCollection parameters = new SPParamCollection();
            parameters.Add(new SPParam("url", "url", true, null, new SPUrlValidator(), "Please specify the web application url."));
            parameters.Add(new SPParam("allowaccesstowebpartcatalog", "a", false, "true", new SPTrueFalseValidator()));

            StringBuilder sb = new StringBuilder();
            sb.Append(
                "\r\n\r\nSets whether or not to allow access to the online web part gallery.\r\n\r\nParameters:");
            sb.Append("\r\n\t-url <web application>");
            sb.Append("\r\n\t[-allowaccesstowebpartcatalog <true (default) | false>]");

            Init(parameters, sb.ToString());
        }

        #region ISPStsadmCommand Members

        /// <summary>
        /// Gets the help message.
        /// </summary>
        /// <param name="command">The command.</param>
        /// <returns></returns>
        public override string GetHelpMessage(string command)
        {
            return HelpMessage;
        }

        /// <summary>
        /// Runs the specified command.
        /// </summary>
        /// <param name="command">The command.</param>
        /// <param name="keyValues">The key values.</param>
        /// <param name="output">The output.</param>
        /// <returns></returns>
        public override int Run(string command, StringDictionary keyValues, out string output)
        {
            output = string.Empty;

            InitParameters(keyValues);           

            bool allowAccessToWebPartCatalog = bool.Parse(Params["allowaccesstowebpartcatalog"].Value);
            
            SPWebApplication wa = SPWebApplication.Lookup(new Uri(Params["url"].Value.TrimEnd('/')));

            wa.AllowAccessToWebPartCatalog = allowAccessToWebPartCatalog;

            wa.Update();

            return 1;
        }

        #endregion
    }

}
