using System;
using System.Collections.Generic;
using System.Collections.Specialized;
using System.Reflection;
using System.Text;
using System.Web.UI.WebControls.WebParts;
using System.Xml;
using Edfinancial.SharePoint.STSADM.Commands.SPValidators;
using Microsoft.SharePoint;
using Microsoft.SharePoint.Utilities;
using Microsoft.SharePoint.WebPartPages;
using WebPart=System.Web.UI.WebControls.WebParts.WebPart;

namespace Edfinancial.SharePoint.STSADM.Commands.WebParts
{
    public class SetWebPartState : SPOperation
    {
        /// <summary>
        /// Initializes a new instance of the <see cref="SetWebPartState"/> class.
        /// </summary>
        public SetWebPartState()
        {
            SPParamCollection parameters = new SPParamCollection();
            parameters.Add(new SPParam("url", "url", true, null, new SPNonEmptyValidator(), "Please specify the page url."));
            parameters.Add(new SPParam("id", "id", false, null, new SPNonEmptyValidator(), "Please specify the web part ID."));
            parameters.Add(new SPParam("title", "t", false, null, new SPNonEmptyValidator(), "Please specify the web part title."));
            parameters.Add(new SPParam("delete", "del", false, null, null));
            parameters.Add(new SPParam("open", "open", false, null, null));
            parameters.Add(new SPParam("close", "close", false, null, null));
            parameters.Add(new SPParam("add", "a"));
            parameters.Add(new SPParam("assembly", "asm", false, null, new SPNonEmptyValidator()));
            parameters.Add(new SPParam("typename", "type", false, null, new SPNonEmptyValidator()));
            parameters.Add(new SPParam("zone", "z", false, null, new SPNonEmptyValidator(), "Please specify the zone to move to."));
            parameters.Add(new SPParam("zoneindex", "zi", false, null, new SPIntRangeValidator(0, int.MaxValue)));
            parameters.Add(new SPParam("properties", "props", false, null, new SPNonEmptyValidator()));
            parameters.Add(new SPParam("propertiesfile", "propsfile", false, null, new SPFileExistsValidator()));
            parameters.Add(new SPParam("publish", "p", false, null, null));

            StringBuilder sb = new StringBuilder();
            sb.Append("\r\n\r\nOpens, Closes, Adds, or Deletes a web part on a page.\r\n\r\nParameters:");
            sb.Append("\r\n\t-url <web part page URL>");
            sb.Append("\r\n\t{-id <web part ID> |\r\n\t -title <web part title>}");
            sb.Append("\r\n\t{-delete |\r\n\t -open |\r\n\t -close |\r\n\t -add}");
            sb.Append("\r\n\t[-assembly <assembly name>]");
            sb.Append("\r\n\t[-typename <type name>]");
            sb.Append("\r\n\t[-zone <zone ID>]");
            sb.Append("\r\n\t[-zoneindex <zone index>]");
            sb.Append("\r\n\t{[-properties <comma separated list of key value pairs: \"Prop1=Val1,Prop2=Val2\">] | ");
            sb.Append("\r\n\t [-propertiesfile <path to a file with xml property settings (<Properties><Property Name=\"Name1\">Value1</Property><Property Name=\"Name2\">Value2</Property></Properties>)>]}");
            sb.Append("\r\n\t[-publish]");

            Init(parameters, sb.ToString());
        }

        #region ISPStsadmCommand Members

        /// <summary>
        /// Gets the help message.
        /// </summary>
        /// <param name="command">The command.</param>
        /// <returns></returns>
        public override string GetHelpMessage(string command)
        {
            return HelpMessage;
        }

        /// <summary>
        /// Runs the specified command.
        /// </summary>
        /// <param name="command">The command.</param>
        /// <param name="keyValues">The key values.</param>
        /// <param name="output">The output.</param>
        /// <returns></returns>
        public override int Run(string command, StringDictionary keyValues, out string output)
        {
            output = string.Empty;

            InitParameters(keyValues);

            string url = Params["url"].Value;

            using (SPSite site = new SPSite(url))
            using (SPWeb web = site.OpenWeb()) // The url contains a filename so AllWebs[] will not work unless we want to try and parse which we don't
            {
                SPFile file = web.GetFile(url);

                // file.Item will throw "The object specified does not belong to a list." if the url passed
                // does not correspond to a file in a list.

                if (file.InDocumentLibrary)
                {
                    if (!Utilities.IsCheckedOut(file.Item) || !Utilities.IsCheckedOutByCurrentUser(file.Item))
                        file.CheckOut();
                        // If it's checked out by another user then this will throw an informative exception so let it do so.
                }

                string displayTitle = string.Empty;
                WebPart wp = null;
                SPLimitedWebPartManager manager = null;
                try
                {
                    if (!Params["add"].UserTypedIn)
                    {
                        if (Params["id"].UserTypedIn)
                        {
                            wp = Utilities.GetWebPartById(web, url, Params["id"].Value, out manager);
                        }
                        else
                        {
                            wp = Utilities.GetWebPartByTitle(web, url, Params["title"].Value, out manager);
                            if (wp == null)
                            {
                                throw new SPException(
                                    "Unable to find specified web part.  Try specifying the -id parameter instead (use enumpagewebparts to get the ID)");
                            }
                        }

                        if (wp == null)
                        {
                            throw new SPException("Unable to find specified web part.");
                        }

                        // Set this so that we can add it to the check-in comment.
                        displayTitle = wp.DisplayTitle;

                        if (Params["delete"].UserTypedIn)
                            manager.DeleteWebPart(wp);
                        else if (Params["close"].UserTypedIn)
                            manager.CloseWebPart(wp);
                        else if (Params["open"].UserTypedIn)
                            manager.OpenWebPart(wp);


                        if (!Params["delete"].UserTypedIn)
                        {
                            string zoneID = manager.GetZoneID(wp);
                            int zoneIndex = wp.ZoneIndex;

                            if (Params["zone"].UserTypedIn)
                                zoneID = Params["zone"].Value;
                            if (Params["zoneindex"].UserTypedIn)
                                zoneIndex = int.Parse(Params["zoneindex"].Value);

                            manager.MoveWebPart(wp, zoneID, zoneIndex);

                            if (Params["properties"].UserTypedIn || Params["propertiesfile"].UserTypedIn)
                            {
                                Dictionary<string, string> props =
                                    GetPropertiesArray(Params["properties"].Value, Params["propertiesfile"].Value);
                                SetWebPartProperties(wp, props);
                            }
                            manager.SaveChanges(wp);
                        }
                    }
                    else
                    {
                        manager = web.GetLimitedWebPartManager(url, PersonalizationScope.Shared);
                        wp = AddWebPart(Params["assembly"].Value, Params["typename"].Value);

                        if (Params["title"].UserTypedIn)
                            wp.Title = Params["title"].Value;

                        if (Params["properties"].UserTypedIn || Params["propertiesfile"].UserTypedIn)
                        {
                            Dictionary<string, string> props =
                                GetPropertiesArray(Params["properties"].Value, Params["propertiesfile"].Value);
                            SetWebPartProperties(wp, props);
                        }
                        string zoneID = Params["zone"].Value;
                        int zoneIndex = int.Parse(Params["zoneindex"].Value);

                        displayTitle = wp.DisplayTitle;
                        manager.AddWebPart(wp, zoneID, zoneIndex);
                    }

                }
                finally
                {
                    if (manager != null)
                    {
                        manager.Web.Dispose();
                        manager.Dispose();
                    }
                    if (wp != null)
                        wp.Dispose();

                    if (file.InDocumentLibrary && Utilities.IsCheckedOut(file.Item))
                        file.CheckIn("Checking in changes to page layout due to state change of web part " + displayTitle);

                    if (Params["publish"].UserTypedIn && file.InDocumentLibrary)
                    {
                        file.Publish("Publishing changes to page layout due to state change of web part " + displayTitle);
                        if (file.Item.ModerationInformation != null)
                        {
                            file.Approve("Approving changes to page layout due to state change of web part " + displayTitle);
                        }
                    }
                }
            }

            return 1;
        }

        /// <summary>
        /// Validates the specified key values.
        /// </summary>
        /// <param name="keyValues">The key values.</param>
        public override void Validate(StringDictionary keyValues)
        {
            SPMultiParameterValidator.Validate(Params, new string[] {"add", "close", "open", "delete"}, 1, 1);

            if (Params["add"].UserTypedIn)
            {
                Params["assembly"].IsRequired = true;
                Params["typename"].IsRequired = true;
                Params["zone"].IsRequired = true;
                Params["zoneindex"].IsRequired = true;
                Params["id"].Enabled = false;
                Params["id"].IsRequired = false;
                Params["title"].IsRequired = true;
            }
            else
            {
                SPBinaryParameterValidator.Validate("id", Params["id"].Value, "title", Params["title"].Value);
            }
            if (Params["properties"].UserTypedIn)
            {
                string[] props = Params["properties"].Value.Split(',');
                foreach (string prop in props)
                {
                    if (prop.Split('=').Length != 2)
                        throw new SPSyntaxException(
                            "The format of the properties parameter is incorrect: \"prop1=val1,prop2=val2\"");
                }
            }
            if (Params["properties"].UserTypedIn && Params["propertiesfile"].UserTypedIn)
                throw new SPSyntaxException("The properties parameter and propertiesfile parameters are incompatible.");

            base.Validate(keyValues);
        }

        #endregion

        /// <summary>
        /// Gets the properties array.
        /// </summary>
        /// <param name="properties">The properties.</param>
        /// <param name="propertiesFile">The properties file.</param>
        /// <returns></returns>
        private static Dictionary<string, string> GetPropertiesArray(string properties, string propertiesFile)
        {
            if (string.IsNullOrEmpty(properties) && string.IsNullOrEmpty(propertiesFile))
                return null;

            Dictionary<string, string> props = new Dictionary<string, string>();
            if (!string.IsNullOrEmpty(properties))
            {
                foreach (string s in properties.Split(','))
                {
                    string[] prop = s.Split('=');
                    props.Add(prop[0].Trim(), prop[1].Trim());
                }
            }
            else
            {
                XmlDocument xmlDoc = new XmlDocument();
                xmlDoc.Load(propertiesFile);
                foreach (XmlElement propElement in xmlDoc.DocumentElement.ChildNodes)
                {
                    props.Add(propElement.Attributes["Name"].Value, propElement.InnerText.Trim());
                }
            }
            return props;
        }

        /// <summary>
        /// Adds the web part.
        /// </summary>
        /// <param name="assemblyName">Name of the assembly.</param>
        /// <param name="typeName">Name of the type.</param>
        /// <returns></returns>
        private static WebPart AddWebPart(string assemblyName, string typeName)
        {
            //now try loading the assembly
            Assembly wpAsm;
            WebPart xWp;
            try
            {
                wpAsm = Assembly.Load(assemblyName);
            }
            catch (Exception asmEx)
            {
                //some error logging here about invalid assembly name
                throw new SPException("Error loading assembly name " + assemblyName + ":\r\nt" + Utilities.FormatException(asmEx));
            }

            if (wpAsm == null)
                throw new SPException("Error loading assembly name " + assemblyName);

            //try creating an instance of the class
            try
            {
                xWp = (WebPart)wpAsm.CreateInstance(typeName);
            }
            catch (Exception instEx)
            {
                //some error logging here about invalid class name
                throw new SPException("Error creating instance of class " + typeName + ":\r\n" + Utilities.FormatException(instEx));
            }

            if (xWp == null)
                throw new SPException("Error creating instance of class " + typeName);

            return xWp;
        }


        /// <summary>
        /// Sets the web part properties.
        /// </summary>
        /// <param name="xWp">The x wp.</param>
        /// <param name="properties">The properties.</param>
        private static void SetWebPartProperties(WebPart xWp, Dictionary<string, string> properties)
        {
            //check to see if there are any properties; if there are zero it won't say
            //zero, it will say null (unlike vb.net)
            if (properties == null)
                return;

            foreach (string prop in properties.Keys)
            {
                try
                {
                    object val = properties[prop];
                    PropertyInfo propInfo = xWp.GetType().GetProperty(prop);
                    if (propInfo == null)
                        throw new SPException(string.Format("Unable to set property '{0}'.  The property could not be found.", prop));
                    
                    val = Convert.ChangeType(val, propInfo.PropertyType);
                    if (val == null)
                        throw new SPException(string.Format("Unable to convert '{0}' to type '{1}'", properties[prop], propInfo.PropertyType));

                    propInfo.SetValue(xWp, val, null);
                }
                catch (Exception ex)
                {
                    throw new SPException(string.Format("Error setting property value {0} to {1}:\r\n{2}", prop, properties[prop], Utilities.FormatException(ex)));
                }
            }
        }
    }
}
